<?php

namespace InSegment\WorkersFramework\Queues;

use InSegment\WorkersFramework\Drivers\RabbitMq;
use Shavshukov\EventAppBase\Interfaces\ValidatableInterface;

class ConnectionsConfig
{
    const AVAILABLE_DRIVERS = ['rabbit', 'redis'];

    /**
     * Connection options
     *
     * @var \Shavshukov\EventAppBase\Interfaces\ModelInterface[]
     */
    protected $connectionOptions = [];
    
    /**
     * Connectors
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueConnectorInterface[] 
     */
    protected $connectors = [];
    
    /**
     * Constructor
     * 
     * @param array $config
     */
    public function __construct(array $config)
    {
        foreach ($config as $connectionName => $connectionConfig) {
            if (!in_array($connectionConfig['driver'] ?? null, self::AVAILABLE_DRIVERS)) {
                $printAvailableDrivers = implode(', ', self::AVAILABLE_DRIVERS);
                throw new \Exception("Every connection must be configured with one of these drivers: {$printAvailableDrivers}. It is not configured for key {$connectionName}");
            }
            
            if (!is_array($connectionConfig['queues'] ?? null) || !count($connectionConfig['queues'])) {
                throw new \Exception("Every connection must be assigned an array of queues, non found for {$connectionName}");
            }
            
            switch($connectionConfig['driver']) {
                case 'rabbit':
                    $this->addRabbitMqConnector($connectionName, $connectionConfig);
                break;
                   
                case 'redis':
                    $this->addRedisConnector($connectionName, $connectionConfig);
                break;
            }
        }
    }
    
    /**
     * Get connectors
     * 
     * @return \InSegment\WorkersFramework\Interfaces\QueueConnectorInterface[]
     */
    public function getConnectors(): array
    {
        return $this->connectors;
    }
    
    /**
     * Get connection options for conneciton
     * 
     * @param string $connectionName
     * @return \Shavshukov\EventAppBase\Interfaces\ModelInterface|null
     */
    public function getConnectionOptions(string $connectionName)
    {
        return $this->connectionOptions[$connectionName] ?? null;
    }
    
    /**
     * Add connector for rabbit driver
     * 
     * @param string $connectionName
     * @param array $connectionConfig
     * @return null
     */
    protected function addRabbitMqConnector(string $connectionName, array $connectionConfig)
    {
        $connectionOptions = RabbitMq\ConnectionOptions::from($connectionConfig);
        $this->validateOptions($connectionName, $connectionOptions);
        $this->connectionOptions[$connectionName] = $connectionOptions;
        $this->connectors[$connectionName] = new RabbitMq\Connector($connectionOptions);
    }
    
    /**
     * Add connector for rabbit driver
     * 
     * @param string $connectionName
     * @param array $connectionConfig
     * @return null
     */
    protected function addRedisConnector(string $connectionName, array $connectionConfig)
    {
        // TODO: implement
    }
    
    /**
     * Check options supplied for a connection driver
     * 
     * @param string $connectionName
     * @param \Shavshukov\EventAppBase\Interfaces\ValidatableInterface $connectionOptions
     * @return null
     * @throws \Exception
     */
    protected function validateOptions(string $connectionName, ValidatableInterface $connectionOptions)
    {
        $validation = $connectionOptions->validate();
        
        if (!$validation->isValid()) {
            $printNotFoundFields = implode(', ', $validation->getInvalidFields());
            throw new \Exception("Mandatory fields not found for {$connectionName} connection config: {$printNotFoundFields}\n");
        }
    }
    
}
